import { computed, defineComponent } from "vue";
import { RouterView } from "vue-router";
import TheError from "../components/The-error";
import { setUserInfo, storeData } from "../store";
import { getQuery } from "../utils/queryString";
import { employeeQueryUserInfo, studentQueryUserInfo, teacherQueryUserInfo } from "./api";
import {
	GlobalThemeOverrides,
	NConfigProvider,
	NDialogProvider,
	NMessageProvider,
	NNotificationProvider,
	darkTheme,
} from "naive-ui";
import { lighten } from "../utils";

export default defineComponent({
	name: "App",
	setup() {
		const appTheme = "#198CFE";
		const lightenStr = lighten(appTheme, 6);
		const themeOverrides: GlobalThemeOverrides = {
			common: {
				primaryColor: appTheme,
				primaryColorHover: lightenStr,
				primaryColorPressed: lightenStr,
			},
			LoadingBar: {
				colorLoading: appTheme,
			},
		};
		const query: any = getQuery();
		/** 获取用户信息 */
		const getUserInfo = async () => {
			// const a = await request.get(`/student/queryUserInfo`)
			// console.log(a)
			if (storeData.platformType === "WEB") {
				return await employeeQueryUserInfo();
			} else if (storeData.platformType === "TEACHER") {
				return await teacherQueryUserInfo();
			}
			return await studentQueryUserInfo();
		};
		const setUser = async () => {
			const res = await getUserInfo();
			const { student } = res?.data || {};
			setUserInfo(student);
			// console.log("🚀 ~ res:", res);
		};
		// onBeforeMount(() => {
		// 	if (query.Authorization) {
		// 		setToken(query.Authorization);
		// 	}
		// 	setUser();
		// 	setBehaviorId(getRandomKey())
		// 	setCampId(query.campId || '')
		// });
		// onMounted(() => {
		// 	const _loading = document.getElementById("loading")
		// 	_loading && (document.body.removeChild(_loading))
		// })

		const inited = computed(() => {
			return true;
			// return storeData.status === "login";
		});
		return () => (
			<NConfigProvider inlineThemeDisabled themeOverrides={themeOverrides} abstract>
				<NDialogProvider>
					<NNotificationProvider>
						<NMessageProvider max={1} theme={darkTheme.Message}>
							{storeData.status === "error" ? <TheError /> : inited.value ? <RouterView /> : null}
						</NMessageProvider>
					</NNotificationProvider>
				</NDialogProvider>
			</NConfigProvider>
		);
	},
});
